var global_annos = [];
const MAX_TAG_LENGTH = '150';
var annoObj = {};
jQuery(document).ready(function () {
  // kyle added to handle the issue View tab and Add/Edit Annotation have same URL
  var can_read_annotations = false;
  var perms = drupalSettings.recogito_integration.permissions;
  if (perms['recogito view annotations'] && !window.location.pathname.includes('recogito_integration')) {

    if(jQuery('.openseadragon-canvas').length > 0){ //check for OpenSeadragon
      setTimeout(awaitOpenSeadragonAnnotorious, 300, perms);
    }
    //need to seperate this!!
    if (drupalSettings.recogito_integration.admin_view_mode)
      initializeAdmin(perms);
    else
      initializeNonAdmin(perms);
  }
});
/**
 * Converts a hexadecimal color code into the corresponding RGBA string. 
 * Credit:  https://stackoverflow.com/a/21648508
 * @param {string} hex the hex color code
 * @returns the RGBA representation of hex
 */
function hexToRgbA(hex, transparency = 1){
  let c;
  transparency === null ? 0 : transparency;
  if(/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)){
      c= hex.substring(1).split('');
      if(c.length== 3){
          c= [c[0], c[0], c[1], c[1], c[2], c[2]];
      }
      c= '0x'+c.join('');
      return 'rgba('+[(c>>16)&255, (c>>8)&255, c&255].join(',')+`,${transparency})`;
  }
  throw new Error('Bad Hex');
}

/**
 * Cancels the selection of all image annotations (simple & OSD)
 */
function cancelAllSelections(){
  for (var i = 0; i < global_annos.length; i++) {
    if(global_annos[i] !== undefined) global_annos[i].cancelSelected();
  }
}

/**
 * Setup annotations for simple images (those not in an OpenSeadragon viewer)
 * @param image : Object HTMLImageElement
 * @param perms : assigned permission config
 * @param node_num : int the node to which the image belongs
 */
function initSimpleImageAnnotation(image, perms, node_num){
  var user_data = drupalSettings.recogito_integration.user_data;
  var customAttributeName = drupalSettings.recogito_integration.attach_attribute_name;
  var range = drupalSettings.recogito_integration.annotation_range;
  var strings = drupalSettings.recogito_integration.taxonomy_terms;
  var default_term = drupalSettings.recogito_integration.default_term;
  var readOnly1 = (!perms['recogito create annotations'] &&
      !perms['recogito edit annotations'] &&
      !perms['recogito delete annotations'] &&
      !perms['recogito edit own annotations'] &&
      !perms['recogito delete own annotations'])
  var imgElement = image;
  var anno = Annotorious.init({
    image: imgElement,
    //  readOnly: readOnly1 || window.location.search !== "?mode=annotation"
    readOnly: readOnly1 || !window.location.search.includes('?mode=annotation'),
    widgets: [
      'COMMENT',
      {widget: 'TAG', vocabulary: strings}
    ],
    relationVocabulary: ['isRelated', 'isPartOf', 'isSameAs ']
  });
  // var anno = Annotorious.init({image: imgElement});
  anno.setAuthInfo({'id': user_data.id, 'displayName': user_data.displayName});


  // window.location.hostname + "/modules/recogito_integration/recogito_integration_functions.php"
  var page_url = window.location.pathname;
  var curr_node = window.location.pathname.split('/')[2] == node_num;
  if (!curr_node) page_url = '/node/' + node_num;
 // !global_annos.includes(anno) && global_annos.push(anno);
  jQuery.ajax({
    type: "GET",
    url: "/recogito_integration/get",
    dataType: 'json',
    headers: {
      'pageurl': page_url//'/node/' + node_num
    },

    success: function (data) {
      console.log(data);
      for (annotation in data) {

        w3c = convert_annotation_w3c(data[annotation]);
        if (w3c.type !== "Annotation" && imgElement["src"] == w3c["target"]["source"]) {
          anno.addAnnotation(w3c);
        }
      }

    },
    error: function (xhr, status, error) {
      alert(xhr.responseText);
    }
  });

  anno.on('selectAnnotation', function (annotation) {
    if (!anno.ready) setTimeout(addAccessibilityLabel, 10);
    highlightAnnotatedContent(annotation);
    if (readOnly1 || !window.location.search.includes('?mode=annotation'))setTimeout(fixList, 25);
  });

  anno.on('createAnnotation', function (annotation) {

    // add a fix for 500 error when add diacritics (.ie: Öçè) to a comment or reply
    //annotation.body[0].value = encode_utf8(annotation.body[0].value);
    for (var i = 0; i < annotation.body.length; i++) {
      annotation.body[i].value = encode_utf8(annotation.body[i].value);
    }

    // check with John
    //create_annotation(annotation);
    if (!curr_node)
      create_annotation(annotation, node_num, "IMG");
    else
      create_annotation(annotation, -1, "IMG");
  });

  anno.on('updateAnnotation', function (annotation, previous) {

    console.log(annotation);
    console.log(previous);
    if (JSON.stringify(annotation) !== JSON.stringify(previous)) update_annotation(annotation, previous); //only update if there is actually a change (reduces unneccessary refresh)
  });

  anno.on('deleteAnnotation', function (annotation) {
    for (var i = 0; i < annotation.body.length; i++) {
      annotation.body[i].value = encode_utf8(annotation.body[i].value);
    }
    delete_annotation(annotation);
  });
}


function getParentNode(node_num){
  var xmlHttp = new XMLHttpRequest();
  var url = window.location.origin + '/node/' + node_num + '?_format=json';//window.location.pathname + '?_format=json';
  xmlHttp.open( "GET", url, false ); // false for synchronous request
  xmlHttp.send( null );
  var page_json = JSON.parse(xmlHttp.responseText);
  if (page_json['field_part_of'].length)
    return page_json['field_part_of'][0]['target_id'];
  else return null;
}
/**
 * Setup annotation for text
 * @param perms : assigned permission config
 */
function initTextAnnotation(perms) {
  var user_data = drupalSettings.recogito_integration.user_data;
  var customAttributeName = drupalSettings.recogito_integration.attach_attribute_name;
  var range = drupalSettings.recogito_integration.annotation_range;
  var strings = drupalSettings.recogito_integration.taxonomy_terms;
  var default_term = drupalSettings.recogito_integration.default_term;
  var readOnly = (!perms['recogito create annotations'] &&
      !perms['recogito edit annotations'] &&
      !perms['recogito delete annotations'] &&
      !perms['recogito edit own annotations'] &&
      !perms['recogito delete own annotations'])

  // When in View mode (View tab is active), Show annotation in readonly
  /* if (readOnly || !window.location.search.includes('?mode=annotation')) {
     jQuery("article > div.node__content > div.field--name-body").click(function (e) {
       // Kyle added this to enforce hide the annotation for View Tab
       jQuery('.r6o-editor').attr('style','display:none !important');
       if (e.target.tagName.toLowerCase() === 'span' && (jQuery(e.target).attr('class') === "r6o-annotation")) {
         jQuery('.r6o-editor').show();

         // hide reply and tag textfield, modify footer buttons
         setTimeout(() => {
           jQuery('.r6o-editor').find("div.r6o-widget.comment.editable").hide();
           jQuery('.r6o-editor').find("div[role='combobox']").hide();
           jQuery("button.r6o-btn").each(function(index) {
             if (jQuery(this).text() === "Ok")
               jQuery(this).hide();
             else
               jQuery(this).html("Ok");
           });
         }, 10);

       }
     });
   } */

  // visually set Annotation tab item enabled
  jQuery("ul.primary > li").each(function( index ) {
    jQuery(this).removeClass('is-active');
  });

  // Kyle added: special usecase : if Custom DOM mode is on, look and highlight and enable annotation for that DOM only
  var attach_element = [-1];
  if (0){//range === "limited" && customAttributeName.length > 0) {
  }
  else {
    // change background for annotated area:
    if (window.location.search.includes("?mode=annotation")){
      if (arguments[2] !== undefined)
        arguments[2].style.backgroundColor = '#dfeaff'; //change background of annotatable content
    }
    var attach_element = jQuery("article > div.node__content");
    for (var j = 0; j < customAttributeName.length; j++) {
      var element = jQuery(customAttributeName[j]);
      if (customAttributeName[j].startsWith("#")) {
        if (window.location.search == "?mode=annotation") {
          element.style.backgroundColor = '#dfeaff';
        }
        attach_element.push(element[0]);
      }
      else {
        for (var k = 0; k < element.length; k++) {
          if (window.location.search == "?mode=annotation") {
            element[k].style.backgroundColor = '#dfeaff';
          }
          attach_element.push(element[k]);
        }
      }
    }
  }
  // check  annotation are allow to be enable for DOM or content type. If not, display warning
  if (attach_element[0] !== -1 ) {
    // need [0] because selector returns an array instead of object
    if (arguments.length !== 3){
      for (var i = 0; i < attach_element.length; i++) {
        var text_anno = Recogito.init({
          content: attach_element[i], // Element id or DOM node to attach to
          allowEmpty: true,
          locale: 'auto',
          widgets: [
            'COMMENT',
            {widget: 'TAG', vocabulary: strings}
          ],
          relationVocabulary: ['isRelated', 'isPartOf', 'isSameAs '],
          readOnly: readOnly && !window.location.search.includes("?mode=annotation") //John added this
        });
        text_anno.domHookEle = attach_element[i];
        text_anno.setAuthInfo({'id': user_data.id, 'displayName': user_data.displayName});

        if (default_term != -1) { // ignore when no default tag is selected
          jQuery( ".node__content" ).bind('DOMSubtreeModified', function (e) {
            if (e.target.tagName === "SPAN" && e.target.hasAttribute("data-id") === false) {
              cancelAllSelections();
              setTimeout(setDefaultTerm, 10);
              //setTimeout(addAccessibilityLabel, 10);
              return;
            }
          });
        }

        text_anno.on('selectAnnotation', function (annotation) {

          // TODO: check if there is preset configuration ready before intial Recogito JS annotation
          if (drupalSettings.recogito_integration.initial_setup){
            highlightAnnotatedContent(annotation);
            if (text_anno.readOnly == undefined || !text_anno.readOnly) setTimeout(addAccessibilityLabel, 15);
            if (readOnly || !window.location.search.includes("?mode=annotation") ) {
              setTimeout(function () {
                var tag_lists = jQuery(".r6o-widget.r6o-tag");
                for (let i = 0; i < tag_lists.length; i++) {
                  if (tag_lists[i].getElementsByClassName('r6o-taglist').length == 0)
                    tag_lists[i].style.display = 'none';
                }
                /*if (!jQuery(".r6o-label").length) {
                  jQuery(".r6o-widget.r6o-tag").hide();
                }*/
              }, 25);
              setTimeout(fixList, 25);
            }
            else {
              cancelAllSelections();
              if(readOnly || !window.location.search.includes("?mode=annotation")) setTimeout(fixList, 25);
            }
          }
          else{
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          }
        });
        text_anno.on('createAnnotation', function (annotation) {
          if (default_term != -1) { // ignore when no default tag is selected
            // set "footnote" as default vocabulary
            var tmp = annotation.body[0];
            annotation.body.push({
              created: tmp.created,
              creator: tmp.creator,
              modified: tmp.modified,
              purpose: "tagging",
              type: tmp.type,
              value: default_term,
            });
          }
          // TODO: check if there is preset configuration ready before intial Recogito JS annotation
          if (drupalSettings.recogito_integration.initial_setup === false)
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          else if (perms['recogito create annotations'] === false)
            alert("Your annotation won't be saved because you don't have permission to create annotation for this content.")
          else{
            create_annotation(annotation, -1, text_anno.domHookEle);
          }
        });

        text_anno.on('updateAnnotation', function (annotation, previous) {
          console.log("Test");
          // TODO: check if there is preset configuration ready before intial Recogito JS annotation
          if (drupalSettings.recogito_integration.initial_setup === false)
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          else if (perms['recogito edit annotations'] === false)
            alert("Your annotation won't be saved because you don't have permission to update this annotation of this content.")
          else
          {
            if (JSON.stringify(annotation) !== JSON.stringify(previous))
              update_annotation(annotation, previous);
          }
        });

        text_anno.on('deleteAnnotation', function (annotation) {
          // TODO: check if there is preset configuration ready before intial Recogito JS annotation

          if (drupalSettings.recogito_integration.initial_setup === false)
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          else if (perms['recogito delete annotations'] === false)
            alert("Your annotation won't be saved because you don't have permission to update this annotation of this content.")
          else
            delete_annotation(annotation);
        });
      }
      getAnnotations(readOnly);
    }
    else{
      var working_node = arguments[1];
      for (var i = 0; i < attach_element.length; i++) {
        var text_anno = Recogito.init({
          content: attach_element[i], // Element id or DOM node to attach to
          allowEmpty: false,  //changed to false by John
          locale: 'auto',
          widgets: [
            'COMMENT',
            {widget: 'TAG', vocabulary: strings}
          ],
          relationVocabulary: ['isRelated', 'isPartOf', 'isSameAs '],
          readOnly: readOnly || !window.location.search.includes("?mode=annotation")  //John added this
        });
        text_anno.setAuthInfo({'id': user_data.id, 'displayName': user_data.displayName});
        text_anno.domHookEle = attach_element[i];
        annoObj[convert_dom_element_object(text_anno.domHookEle)] = text_anno;
        text_anno.annotations = [];
        // check with John
        //getAnnotations(text_anno);
        // set default term (if applicable) as default tag
        if (default_term != -1) { // ignore when no default tag is selected
          jQuery( ".node__content" ).bind('DOMSubtreeModified', function (e) {
            if (e.target.tagName === "SPAN" && e.target.hasAttribute("data-id") === false) {
              cancelAllSelections();
              setTimeout(setDefaultTerm, 10);
              //setTimeout(addCountWords, 25);
            // setTimeout(addCountableTag, 25);
              setTimeout(addMaxLength, 25);

              return;
            }
          });
        }
        text_anno.on('selectAnnotation', function (annotation) {
  //        resizeList();
          // TODO: check if there is preset configuration ready before intial Recogito JS annotation
          if (drupalSettings.recogito_integration.initial_setup){
            window.location.search.includes("?mode=annotation") && setTimeout(addDeleteButton, 30, annotation, text_anno);
            highlightAnnotatedContent(annotation);
            if (text_anno.readOnly == undefined || !text_anno.readOnly) {
              setTimeout(addAccessibilityLabel, 15);

              // only View tab or Front facing show rendered HTML annotation text
              if (window.location.search !== "?mode=annotation") {
                setTimeout(displayAnnotationAsHTML, 20);
              }
            }
            if (readOnly || !window.location.search.includes("?mode=annotation") ) {
              setTimeout(function () {
                var tag_lists = jQuery(".r6o-widget.r6o-tag");
                for (let i = 0; i < tag_lists.length; i++) {
                  if (tag_lists[i].getElementsByClassName('r6o-taglist').length == 0)
                    tag_lists[i].style.display = 'none';
                }
                /*if (!jQuery(".r6o-label").length) {
                  jQuery(".r6o-widget.r6o-tag").hide();
                }*/
              }, 25);
              setTimeout(fixList, 25);
            }
            else {
              cancelAllSelections();
            }

          }
          else{
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          }
        });
        (function(element){
          text_anno.on('createAnnotation', function (annotation) {
            if (default_term != -1) { // ignore when no default tag is selected
  
              // add a fix for 500 error when add diacritics (.ie: Öçè) to a comment or reply
              //annotation.body[0].value = encode_utf8(annotation.body[0].value);
              for (var i = 0; i < annotation.body.length; i++) {
                annotation.body[i].value = encode_utf8(annotation.body[i].value);
              }
  
              // set "footnote" as default vocabulary
              var tmp = annotation.body[0];
              annotation.body.push({
                created: tmp.created,
                creator: tmp.creator,
                modified: tmp.modified,
                purpose: "tagging",
                type: tmp.type,
                value: default_term,
              });
            }
            // TODO: check if there is preset configuration ready before intial Recogito JS annotation
            if (drupalSettings.recogito_integration.initial_setup === false)
              alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
            else if (perms['recogito create annotations'] === false)
              alert("Your annotation won't be saved because you don't have permission to create annotation for this content.")
            else{
              create_annotation(annotation, working_node, element);
            }
          });
        })(attach_element[i]);
        text_anno.on('updateAnnotation', function (annotation, previous) {
          // TODO: check if there is preset configuration ready before intial Recogito JS annotation
          if (drupalSettings.recogito_integration.initial_setup === false)
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          else if (perms['recogito edit annotations'] === false)
            alert("Your annotation won't be saved because you don't have permission to update this annotation of this content.")
          else {
            if (JSON.stringify(annotation) !== JSON.stringify(previous)) { //only update if there is actually a change (reduces unneccessary refresh)
              update_annotation(annotation, previous);
            }
          }
        });
        text_anno.on('deleteAnnotation', function (annotation) {
          // TODO: check if there is preset configuration ready before intial Recogito JS annotation

          if (drupalSettings.recogito_integration.initial_setup === false)
            alert("Your annotation won't be saved because Recogito Annotation has not been setup yet. \n\nPlease setup the configuration at "+window.location.protocol+ "//" +window.location.hostname+"/admin/config/development/recogito_integration");
          else if (perms['recogito delete annotations'] === false)
            alert("Your annotation won't be saved because you don't have permission to update this annotation of this content.")
          else
            delete_annotation(annotation);
        });
      }
      getAnnotations(readOnly, arguments[1]);
    }
  }
  else {
    if (window.location.search == "?mode=annotation") {
      jQuery("article > div.node__content").html('<p><strong>Sorry, the annotation functionality for this content is not available because this content type is not set for annotation OR no preset class or ID can be found in this page\'s body.</strong> </p>' +
          '<p>Please visit <a href="/admin/config/development/recogito_integration"> the configuration</a> for adjustment.</p>');
    }
  }

}

/**
 * Kyle added to set Default Term "Footnote"
 */
function setDefaultTerm() {
  var term = drupalSettings.recogito_integration.default_term;
  var div = jQuery(".r6o-tag").find('div')[0];

  jQuery(div).html(
      '<ul class="r6o-taglist">' +
      '<li>' +
      '<span class="r6o-label">'+term+ '</span>' +
      '</li>' +
      '</ul>'
  );
  //jQuery(".r6o-btn").html("Save");
  //jQuery(".r6o-btn.outline").html("Cancel");

}

/**
 * Kyle added due to request of support HTML tag in annotation text (ie.italic)
 */
function displayAnnotationAsHTML () {
  // Kyle added for support italic for annotation text
  jQuery(".r6o-editable-text").each( function (index) {
    if (jQuery(this).prop('disabled') === true) {
      jQuery(this).before('<div class="readonly-anno">' + jQuery(this).val()+ '</div>');
      jQuery(this).hide();
    }
  });
}


/**
 * Kyle added an UI for textfield character/word counter in case we like to
 * limit the length of comment textfield Currently unused
 */
function modifyImageAnnotation() {
  jQuery(".r6o-autocomplete").find('input').attr("placeholder", "Add a tag (press ENTER for each tag)");
}

/**
 * Kyle added to set Default Term "Footnote"
 */
function setDefaultTermImageAnnotation() {
  var term = drupalSettings.recogito_integration.default_term;
  var div = jQuery(".r6o-tag").find('div')[1];
  jQuery(div).html(
      '<ul class="r6o-taglist">' +
      '<li>' +
      '<span class="r6o-label">'+term+ '</span>' +
      '</li>' +
      '</ul>'
  );
  //jQuery(".r6o-btn").html("Save");
  //jQuery(".r6o-btn.outline").html("Cancel");
}

/**
 * Fixes issues where list overflows in readOnly mode
 */
function fixList(){
  var tag_lists = jQuery('.r6o-widget.r6o-tag');
  for (let i = 0; i < tag_lists.length; i++) {
    tag_lists[i].style.overflow='scroll';
  }
}
/**
 * Kyle added an UI for textfield character/word counter in case we like to
 * limit the length of comment textfield Currently unused
 */
function addCountWords() {
  // add countable feature for
  jQuery('.r6o-editable-text').each(function(index, activeCommentTextfield) {

    if (jQuery(activeCommentTextfield).parent().attr('class') === "r6o-widget comment editable") {
      //if (jQuery().parent().attr('class'))
      if (jQuery('#comment-counter').length === 0)
        jQuery(activeCommentTextfield).after('Limit: <span id="comment-counter">0</span>/200 characters.</strong>');

      jQuery(activeCommentTextfield).simplyCountable({
        counter: '#comment-counter',
        countType: 'characters',
        maxCount: 200,
        strictMax: true,
        countDirection: 'up',
        onOverCount: function (count, countable, counter) {
        },
        onSafeCount: function (count, countable, counter) {
        },
        onMaxCount: function (count, countable, counter) {
        }
      });
    }

  });

}
/**
 * Limits the length of comment tags to MAX_TAG_LENGTH
 */
function addMaxLength(){
  document.querySelectorAll('[role="combobox"]').forEach(function (e) {
    e.firstElementChild.setAttribute('maxlength', MAX_TAG_LENGTH);
  });
}

/**
 * Kyle added an UI for textfield character/word counter in case we like to
 * limit the length of comment textfield Currently unused
 */
function addCountableTag() {


  if (jQuery('#tag-counter').length === 0) {
    jQuery(".r6o-autocomplete").find('input').attr("placeholder", "Add a tag (press ENTER for each tag)");
    jQuery(".r6o-autocomplete").find('input').after('<span class="tag-limit">Tag\'s limit: <span id="tag-counter">0</span>/150 characters.</strong></span>');
  }
  jQuery(".r6o-autocomplete").find('input').simplyCountable({
    counter: '#tag-counteasdfasr',
    countType: 'characters',
    maxCount: 150,
    strictMax: true,
    countDirection: 'up',
    onOverCount: function (count, countable, counter) {
    },
    onSafeCount: function (count, countable, counter) {
    },
    onMaxCount: function (count, countable, counter) {
    }
  });

}

/**
 <<<<<<< HEAD
 * Kyle added to encode annotation text with diacritics
 * @param s
 * @returns {*}
 */
function encode_utf8( s )
{
  return unescape( encodeURIComponent ( s ) );
}

/**
 * Kyle added to decode annotation text with diacritics
 * @param s
 * @returns {string}
 */
function decode_utf8( s )
{
  return decodeURIComponent ( escape( s ) );
}

/**
 * John added to pass accessiblity test for annotations
 =======
 * Adds the required ARIA attributes for annotation editor
 >>>>>>> 6eb988280b6dda8f837e85bc2f514b5a04457010
 */
function addAccessibilityLabel()
{

  //jQuery(".r6o-editable-text").attr("aria-label", "test label");
  var elems = jQuery(".r6o-editable-text");
  for (var i = 0; i < elems.length-1; i++)
  {
    //add aria-label to comments
    elems[i].setAttribute("aria-label", "Comment " + (i+1));
  }
  //add aria-label to reply field
  if(typeof elems[elems.length - 1] !== 'undefined') elems[elems.length - 1].setAttribute("aria-label", "Add a reply");
  //make "aria-labelledby" match the id and add a title
  var loc = jQuery(".r6o-autocomplete").find("input")[0];
  var id = jQuery(loc).attr("id");
  jQuery(loc).attr("aria-labelledby", id);
  jQuery(loc).attr("title", "Add a tag");

  loc = jQuery(".r6o-autocomplete").find("ul")[0];
  id = jQuery(loc).attr("id");
  jQuery(loc).attr("aria-labelledby", id);
}

function addAnnotations(w3c, ro, hookEle, style) {
  annoObj[hookEle].addAnnotation(w3c, ro);
  jQuery(`[data-id='${w3c.id}']`).css({
    'background': hexToRgbA(style.background, style.background_transparency),
    'color': style.text_color,
    'border-bottom': `${style.underline_thickness}px ${style.underline_style} ${hexToRgbA(style.underline_color)}`
  });
}


/**
 * Ajax call to get list of annotation base on node url
 *
 * @param recogito
 * @param node_num : int the node to get the annotations for
 */
function getAnnotations(readOnly, node_num, ro = false) {
  var page_url = window.location.pathname;
  if (typeof node_num !== "boolean" && //make sure a node number is actually being used
      node_num !== window.location.pathname.split('/')[2]) page_url = '/node/' + node_num;//construct the URL is necessary
  jQuery.ajax({
    type: "GET",
    url: "/recogito_integration/get",
    dataType: 'json',
    headers: {
      'pageurl': page_url
    },

    success: function (data) {
      for (annotation in data) {
        w3c = convert_annotation_w3c(data[annotation]);
        if (data[annotation].target_element.length <= 0) {
          addAnnotations(w3c, ro, 'DIV.node__content', data[annotation].style);
        }
        else {
          addAnnotations(w3c, ro, data[annotation].target_element[0].value, data[annotation].style);
        }
      }
    },
    error: function (xhr, status, error) {
      console.log(xhr.responseText);
    }
  });
}

/**
 * Setup annotation for Openseadragon viewer
 *
 * @param viewer : Object viewer object
 * @param perms : assigned permission config
 * @param node_num : int the node to which the OSD viewer belongs
 */
function initOpenSeadragonAnnnotation(viewer, perms, node_num) {
  //////////////////////////////////////////////////////////////////////////////
  var strings = drupalSettings.recogito_integration.taxonomy_terms;
  var user_data = drupalSettings.recogito_integration.user_data;
  var readOnly1 = (!perms['recogito create annotations'] &&
      !perms['recogito edit annotations'] &&
      !perms['recogito delete annotations'] &&
      !perms['recogito edit own annotations'] &&
      !perms['recogito delete own annotations']);

  var image_anno = OpenSeadragon.Annotorious(viewer, {
    readOnly: readOnly1 || !window.location.search.includes("?mode=annotation"),
    widgets: [
      'COMMENT',
      {widget: 'TAG', vocabulary: strings}
    ],
    relationVocabulary: ['isRelated', 'isPartOf', 'isSameAs ']

  });
  image_anno.setAuthInfo({'id': user_data.id, 'displayName': user_data.displayName});

  //for multi page support
  //this was added in newest update, so may be able to remove this in the future with additional configuration
  viewer.addHandler('page', function (){

    //get rid of any annotations that may be open on the current page
    image_anno.cancelSelected();

    //remove all annotation on the viewer
    image_anno.clearAnnotations();

    //get node ID of page, if applicable
    var page_url = node_num == undefined ? '/node/' + node_num : '/node/' + node_num[viewer.currentPage()];

    //add annotations that belong to this page
    jQuery.ajax({
      type: "GET",
      url: "/recogito_integration/get",
      dataType: 'json',
      headers: {
        'pageurl': page_url,
      },

      success: function (data) {
        for (annotation in data) {
          w3c = convert_annotation_w3c(data[annotation]);
          //make sure this annotation belongs here
          if(w3c.type !== "Annotation" && viewer["tileSources"][viewer.currentPage()] == w3c["target"]["source"])
          {
            image_anno.addAnnotation(w3c);
          }
        }
      },
      error: function (xhr, status, error) {
        alert(xhr.responseText);
      }
    });
  });

  //get node ID of page, if applicable
  var page_url = node_num == undefined ? '/node/' + node_num : '/node/' + node_num[viewer.currentPage()];
  jQuery.ajax({
    type: "GET",
    url: "/recogito_integration/get",
    dataType: 'json',
    headers: {
      'pageurl': page_url,
    },

    success: function (data) {
      for (annotation in data) {
        w3c = convert_annotation_w3c(data[annotation]);
        //make sure this annotation belongs here
        if(w3c.type !== "Annotation" && viewer["tileSources"][0] == w3c["target"]["source"])
        {
          image_anno.addAnnotation(w3c);
        }
      }


    },
    error: function (xhr, status, error) {
      alert(xhr.responseText);
    }
  });

  image_anno.on('selectAnnotation', function (annotation) {
    if (!image_anno.readOnly){
      setTimeout(addAccessibilityLabel, 15);
      highlightAnnotatedContent(annotation);
    }
    else if (image_anno.readOnly){
      highlightAnnotatedContent(annotation);
      setTimeout(function (){
        if (!jQuery(".r6o-label").length){
          jQuery(".r6o-widget.r6o-tag").hide();
        }
      }, 25);
      setTimeout(fixList, 25);
    }

  });

  image_anno.on('createAnnotation', function (annotation) {


    // add a fix for 500 error when add diacritics (.ie: Öçè) to a comment or reply
    //annotation.body[0].value = encode_utf8(annotation.body[0].value);
    for (var i = 0; i < annotation.body.length; i++) {
      annotation.body[i].value = encode_utf8(annotation.body[i].value);
    }

    if (node_num.length == undefined) //all pages belong to the same node
    {  
      create_annotation(annotation, node_num, "IMG");
      var parent = getParentNode(node_num);
      if (parent !== null && !viewer.currentPage()) {
        create_annotation(annotation, parent, "IMG");
      }
    }
    else //each page has a different node ID
    {  
      create_annotation(annotation, node_num[viewer.currentPage()], "IMG"); //add the annotation to the page's node
      /* var parent = getParentNode(node_num[0]);
      if (parent !== null && !viewer.currentPage()) { //if page belongs to a book and this is the first page, add anno to book as well
        create_annotation(annotation, parent, viewer.currentPage());
      } */
    }
  });
  image_anno.on('updateAnnotation', function (annotation, previous) {

    if (JSON.stringify(annotation) !== JSON.stringify(previous)) update_annotation(annotation, previous); //only update if there is actually a change (reduces unneccessary refresh)
  });

  image_anno.on('deleteAnnotation', function (annotation) {
    for (var i = 0; i < annotation.body.length; i++) {
      annotation.body[i].value = encode_utf8(annotation.body[i].value);
    }
    delete_annotation(annotation);
  });
}
/**
 * Adds a delete button to a text annotation
 * @param {annotation Object} annotation The annotation object to add the delete button to
 * @param {Recogito} text_anno The Recogito object to which the annotation belongs
 */
function addDeleteButton(annotation, text_anno){
  var footer = document.getElementsByClassName('r6o-footer');

  var delete_button = document.createElement('button');
  delete_button.innerHTML = 'Delete';
  delete_button.className = 'delete-text-annotation r6o-btn';
  /* delete_button.style.position = 'relative';
   delete_button.style.right = '300px';*/
  delete_button.style.cssFloat = 'left';
  delete_button.style.marginLeft =
      delete_button.onclick = function (){
        if (!window.confirm("Are you sure you want to delete this annotation?\nThis action cannot be undone.")) return;
        text_anno.removeAnnotation(annotation);
        delete_annotation(annotation);
      }
  for (let i = 0; i < footer.length; i++) {
    if (footer[i].getElementsByClassName('r6o-btn').length < 3) footer[i].appendChild(delete_button);
  }

}

function convert_dom_element_object(e) {
  var domString = e.tagName;
  var attr = e.attributes;
  for (var i = 0; i < attr.length; i++) {
    if (attr[i].name === "id") {
      domString += `#${attr[i].value}`;
    }
    else if (attr[i].name === "class") {
      domString += `.${attr[i].value.replaceAll(" ", ".")}`;
    }
    else if (attr[i].name === "headers") {
      domString += `[${attr[i].name}="${attr[i].value}"]`
    }
  }
  return domString;
}

/**
 * Create an annotation within Drupal, given W3C data
 *
 * @param a
 * @param node_num : int the on which the annotation is created
 */
function create_annotation(a, node_num, domObj) {
  var page_url;
  var annotation_obj = convert_annotation_object(a);
  if (domObj !== "IMG") {
    annotation_obj.target_element = convert_dom_element_object(domObj);
  }
  else {
    annotation_obj.target_element = domObj;
  }
  // check with John
  node_num != null && node_num > -1 ? page_url = '/node/' + node_num : page_url = window.location.pathname;
  jQuery.ajax({
    type: "POST",
    url: "/recogito_integration/create",
    dataType: 'text',
    headers: {
      'pageurl': page_url,
      'annotationobj': JSON.stringify(annotation_obj)
    },

    success: function (data) {
      location.reload();
    },
    error: function (xhr, status, error) {
      alert("Sorry, unable to create the annotation because of error: \n\n" + error);
    }
  });


}

/**
 * Update an annotation in Drupal, given W3C data
 *
 * @param annotation
 * @param previous
 */
function update_annotation(annotation, previous) {
  // add a fix for 500 error when update annotation with diacritics (.ie: Öçè) in any text field.
  for (var i = 0; i < annotation.body.length; i++) {
    annotation.body[i].value = encode_utf8(annotation.body[i].value);
  }

  var annotation_obj = convert_annotation_object(annotation);
  jQuery.ajax({
    type: "POST",
    url: "/recogito_integration/update",
    dataType: 'text',
    headers: {
      'annotationobj': JSON.stringify(annotation_obj)
    },

    success: function (data) {
      location.reload();
    //  getAnnotations(annotation);
    },
    error: function (xhr, status, error) {
      alert("Sorry, unable to update the annotation because of error: \n\n" + error);
    }
  });
}

/**
 * Delete an annotation from Drupal, given W3C data
 *
 * @param annotation
 */
function delete_annotation(annotation) {
  var annotation_obj = convert_annotation_object(annotation);
  for (let i = 0; i < annotation_obj['textualbodies'].length; i++) {
    //empty out contents before deleting to avoid issues with diacritics
    annotation_obj['textualbodies'][i]['value'] = '';
  }

  jQuery.ajax({
    type: "DELETE",
    url: "/recogito_integration/delete",
    dataType: 'text',
    headers: {
      'annotationobj': JSON.stringify(annotation_obj)
    },

    success: function (data) {
      location.reload();
    },
    error: function (xhr, status, error) {
      //xhr.responseText
      alert("Sorry, unable to delete the annotation because of error: \n\n" + error);
    }
  });
}


/**
 * Convert a W3C annotation object to an object containing the bare minimum of
 * data to store
 *
 * @param a : annotation object
 * @returns {{}}
 */
function convert_annotation_object(a) {
  var annotation_object = {};
  annotation_object.textualbodies = [];
  for (textualbody in a.body) {
    newtextualbody = {};
    newtextualbody.created = a.body[textualbody].created;
    newtextualbody.creator_id = a.body[textualbody].creator.id;
    newtextualbody.creator_name = a.body[textualbody].creator.name;
    newtextualbody.modified = a.body[textualbody].modified;
    newtextualbody.purpose = a.body[textualbody].purpose;
    newtextualbody.value = a.body[textualbody].value;
    annotation_object.textualbodies.push(newtextualbody);
  }
  annotation_object.id = a.id;
  if (Array.isArray(a.target.selector)) { //Textual Annotations
    for (selector in a.target.selector) {
      if (a.target.selector[selector].type == 'TextQuoteSelector') {

        // add a fix for 500 error when select diacritics (.ie: Öçè) in a node
        annotation_object.target_exact = encode_utf8(a.target.selector[selector].exact);


      } else if (a.target.selector[selector].type == 'TextPositionSelector') {
        annotation_object.target_start = a.target.selector[selector].start;
        annotation_object.target_end = a.target.selector[selector].end;
        annotation_object.type = "Annotation";
      }
    }
  } else { //Image annotations
    annotation_object.type = "Selection";
    annotation_object.image_value = a.target.selector.value
    annotation_object.image_source = a.target.source
  }
  annotation_object.title = annotation_object.id;
  return annotation_object;
}

/**
 * Convert an object with only stored data to a W3C object to pass to the
 * library
 *
 * @param annotation_object
 * @returns {{}}
 */
function convert_annotation_w3c(annotation_object) {
  var annotation_w3c = {};
  annotation_w3c['@context'] = "http://www.w3.org/ns/anno.jsonld";
  annotation_w3c.id = annotation_object.id[0].value;
  annotation_w3c.target = {selector: []};
  if (annotation_object.type[0].value == "Selection") {
    annotation_w3c.target.selector = {
      conformsTo: "http://www.w3.org/TR/media-frags/",
      type: "FragmentSelector",
      value: annotation_object.image_value[0].value,
    };
    annotation_w3c.target.source = annotation_object.image_source[0].value;
    annotation_w3c.type = "Selection";
  } else {
    annotation_w3c.type = "Annotation";
    if (annotation_object.target_exact.length > 0 )
      annotation_w3c.target.selector.push({type: "TextQuoteSelector", exact: annotation_object.target_exact[0].value});
    annotation_w3c.target.selector.push({
      type: "TextPositionSelector",
      start: annotation_object.target_start[0].value,
      end: annotation_object.target_end[0].value
    });
  }
  annotation_w3c.body = [];
  for (textualbody in annotation_object.textualbodies) {
    newtextualbody = {};
    newtextualbody.created = annotation_object.textualbodies[textualbody].created[0].value;
    newtextualbody.creator = {
      id: annotation_object.textualbodies[textualbody].creator_id[0].value,
      name: annotation_object.textualbodies[textualbody].creator_name[0].value
    }
    newtextualbody.modified = annotation_object.textualbodies[textualbody].modified[0].value;
    newtextualbody.purpose = annotation_object.textualbodies[textualbody].purpose[0].value;
    newtextualbody.type = "TextualBody";
    newtextualbody.value = annotation_object.textualbodies[textualbody].value[0].value;
    annotation_w3c.body.push(newtextualbody);
  }

  return annotation_w3c;
}
